from typing_extensions import List, TypedDict


class CreateWebhookRequest(TypedDict):
    url: str
    active: bool
    eventTypes: List[str]


class UpdateWebhookRequest(TypedDict):
    url: str
    active: bool
    eventTypes: List[str]



from typing_extensions import List, NotRequired, TypedDict


class File(TypedDict):
    id: str
    name: str
    type: str
    category: str
    metadata: NotRequired[dict[str, str]]
    createdTimestamp: int
    lastUpdatedTimestamp: int
    sizeInBytes: NotRequired[int]


class Files(TypedDict):
    items: List[File]
    paginationToken: NotRequired[str]



from ryft_sdk.client import RyftClient
from ryft_sdk.clients.accounts import AccountsClient
from ryft_sdk.clients.account_links import AccountLinksClient
from ryft_sdk.clients.apple_pay import ApplePayClient
from ryft_sdk.clients.customers import CustomersClient
from ryft_sdk.clients.disputes import DisputesClient
from ryft_sdk.clients.events import EventsClient
from ryft_sdk.clients.files import FilesClient
from ryft_sdk.clients.payment_methods import PaymentMethodsClient
from ryft_sdk.clients.payment_sessions import PaymentSessionsClient
from ryft_sdk.clients.payout_methods import PayoutMethodsClient
from ryft_sdk.clients.payouts import PayoutsClient
from ryft_sdk.clients.persons import PersonsClient
from ryft_sdk.clients.platform_fees import PlatformFeesClient
from ryft_sdk.clients.subscriptions import SubscriptionsClient
from ryft_sdk.clients.transfers import TransfersClient
from ryft_sdk.clients.webhooks import WebhooksClient


class Ryft:
    def __init__(self, secret_key: str):
        self.client = RyftClient(secret_key)
        self.accounts = AccountsClient(self.client)
        self.account_links = AccountLinksClient(self.client)
        self.apple_pay = ApplePayClient(self.client)
        self.customers = CustomersClient(self.client)
        self.disputes = DisputesClient(self.client)
        self.events = EventsClient(self.client)
        self.files = FilesClient(self.client)
        self.payment_methods = PaymentMethodsClient(self.client)
        self.payment_sessions = PaymentSessionsClient(self.client)
        self.payouts = PayoutsClient(self.client)
        self.payout_methods = PayoutMethodsClient(self.client)
        self.persons = PersonsClient(self.client)
        self.platform_fees = PlatformFeesClient(self.client)
        self.subscriptions = SubscriptionsClient(self.client)
        self.transfers = TransfersClient(self.client)
        self.webhooks = WebhooksClient(self.client)



from typing_extensions import NotRequired, TypedDict
from ryft_sdk.models.shipping_details import ShippingDetails


class SubscriptionCustomerRequest(TypedDict):
    id: str


class SubscriptionIntervalRequest(TypedDict):
    unit: str
    count: int
    times: NotRequired[int]


class SubscriptionPriceRequest(TypedDict):
    amount: int
    currency: str
    interval: SubscriptionIntervalRequest


class StatementDescriptorRequest(TypedDict):
    descriptor: str
    city: str


class PaymentSettingsRequest(TypedDict):
    statementDescriptor: StatementDescriptorRequest


class CreateSubscriptionRequest(TypedDict):
    customer: SubscriptionCustomerRequest
    price: SubscriptionPriceRequest
    paymentMethod: SubscriptionCustomerRequest
    description: NotRequired[str]
    billingCycleTimestamp: NotRequired[int]
    metadata: NotRequired[dict[str, str]]
    shippingDetails: NotRequired[ShippingDetails]
    paymentSettings: NotRequired[PaymentSettingsRequest]


class SubscriptionUpdatePriceRequest(TypedDict):
    amount: int
    interval: NotRequired[SubscriptionIntervalRequest]


class UpdateSubscriptionRequest(TypedDict):
    price: NotRequired[SubscriptionUpdatePriceRequest]
    paymentMethod: NotRequired[SubscriptionCustomerRequest]
    description: NotRequired[str]
    billingCycleTimestamp: NotRequired[int]
    metadata: NotRequired[dict[str, str]]
    shippingDetails: NotRequired[ShippingDetails]
    paymentSettings: NotRequired[PaymentSettingsRequest]


class PauseSubscriptionRequest(TypedDict):
    reason: NotRequired[str]
    resumeTimestamp: NotRequired[int]
    unschedule: NotRequired[bool]



from typing import cast, Optional
from ryft_sdk.client import RyftClient
from ryft_sdk.models.events.dto.events import Event, Events


class EventsClient:
    def __init__(self, client: RyftClient):
        self.client = client
        self.path = "events"

    async def get(self, id: str, account: Optional[str] = None) -> Event:
        return cast(Event, self.client.get(f"{self.path}/{id}", account=account))

    async def list(
        self,
        ascending: Optional[bool] = None,
        limit: Optional[int] = None,
        account: Optional[str] = None,
        startsAfter: Optional[str] = None,
    ) -> Events:
        return cast(
            Events,
            self.client.get(
                f"{self.path}",
                {
                    "ascending": ascending,
                    "limit": limit,
                    "startsAfter": startsAfter,
                },
                account=account,
            ),
        )



from typing_extensions import List, TypedDict


class Webhook(TypedDict):
    id: str
    active: bool
    eventTypes: List[str]
    createdTimestamp: int


class Webhooks(TypedDict):
    items: List[Webhook]


class CreatedWebhook(TypedDict):
    secret: str
    id: str
    active: bool
    eventTypes: List[str]
    createdTimestamp: int



from typing import cast, Optional
from ryft_sdk.client import RyftClient
from ryft_sdk.models.platform_fees.dto.platform_fees import (
    PlatformFee,
    PlatformFeeRefunds,
    PlatformFees,
)


class PlatformFeesClient:
    def __init__(self, client: RyftClient):
        self.client = client
        self.path = "platform-fees"

    async def list(
        self, ascending: Optional[bool] = None, limit: Optional[int] = None
    ) -> PlatformFees:
        return cast(
            PlatformFees,
            self.client.get(
                self.path,
                {
                    "ascending": ascending,
                    "limit": limit,
                },
            ),
        )

    async def get(self, id: str) -> PlatformFee:
        return cast(PlatformFee, self.client.get(f"{self.path}/{id}"))

    async def get_refunds(self, id: str) -> PlatformFeeRefunds:
        return cast(PlatformFeeRefunds, self.client.get(f"{self.path}/{id}/refunds"))



from typing import cast
from ryft_sdk.client import RyftClient
from ryft_sdk.models.deleted_resource_resp import DeletedResourceResp
from ryft_sdk.models.payment_methods.dto.payment_methods import PaymentMethod
from ryft_sdk.models.payment_methods.req.payment_methods_req import (
    UpdatePaymentMethodRequest,
)


class PaymentMethodsClient:
    def __init__(self, client: RyftClient):
        self.client = client
        self.path = "payment-methods"

    async def get(self, id: str) -> PaymentMethod:
        return cast(PaymentMethod, self.client.get(f"{self.path}/{id}"))

    async def update(self, id: str, req: UpdatePaymentMethodRequest) -> PaymentMethod:
        return cast(
            PaymentMethod, self.client.patch(f"{self.path}/{id}", cast(dict, req))
        )

    async def delete(self, id: str) -> DeletedResourceResp:
        return cast(DeletedResourceResp, self.client.delete(f"{self.path}/{id}"))



from typing_extensions import List, NotRequired, TypedDict

from ryft_sdk.models.address import Address
from ryft_sdk.models.shipping_details import ShippingDetails


class CustomerDetailsRequest(TypedDict):
    id: NotRequired[str]
    firstName: NotRequired[str]
    lastName: NotRequired[str]
    homePhoneNumber: NotRequired[str]
    mobilePhoneNumber: NotRequired[str]
    metadata: NotRequired[dict[str, str]]


class FeeRequest(TypedDict):
    amount: int


class SplitPaymentRequest(TypedDict):
    id: NotRequired[str]
    accountId: NotRequired[str]
    amount: NotRequired[int]
    description: NotRequired[str]
    fee: NotRequired[FeeRequest]
    metadata: NotRequired[dict[str, str]]


class SplitPaymentsRequest(TypedDict):
    items: List[SplitPaymentRequest]


class PreviousPaymentRequest(TypedDict):
    id: str


class RebillingDetailsRequest(TypedDict):
    amountVeriance: NotRequired[str]
    numberOfDaysBetweenPayments: int
    totalNumberOfPayments: NotRequired[int]
    currentPaymentNumber: NotRequired[int]
    expiry: NotRequired[int]


class OrderDetailRequest(TypedDict):
    reference: str
    name: str
    quantity: int
    unitPrice: int
    taxAmount: int
    totalAmount: int
    discountAmount: NotRequired[int]
    productUrl: NotRequired[str]
    imageUrl: NotRequired[str]


class OrderDetailsRequest(TypedDict):
    items: List[OrderDetailRequest]


class StatementDescriptorRequest(TypedDict):
    descriptor: str
    city: str


class PaymentMethodRequest(TypedDict):
    id: str
    cvc: NotRequired[str]


class CreatePaymentSessionAttemptPaymentRequest(TypedDict):
    paymentMethod: PaymentMethodRequest


class Ts2Py_gdB5O4Lcvi(TypedDict):
    disabled: List[str]


class PaymentSettingsRequest(TypedDict):
    paymentMethodOptions: Ts2Py_gdB5O4Lcvi


class CreatePaymentSessionRequest(TypedDict):
    amount: int
    currency: str
    customerEmail: NotRequired[str]
    customerDetails: NotRequired[CustomerDetailsRequest]
    platformFeee: NotRequired[int]
    splits: NotRequired[SplitPaymentsRequest]
    captureFlow: NotRequired[str]
    paymentType: str
    entryMode: NotRequired[str]
    previousPayment: NotRequired[PreviousPaymentRequest]
    rebillingDetail: NotRequired[RebillingDetailsRequest]
    verifyAccount: NotRequired[bool]
    shippingDetails: NotRequired[ShippingDetails]
    orderDetails: NotRequired[OrderDetailsRequest]
    statementDescriptor: NotRequired[StatementDescriptorRequest]
    metadata: NotRequired[dict[str, str]]
    returnUrl: NotRequired[str]
    attemptPayment: NotRequired[CreatePaymentSessionAttemptPaymentRequest]
    paymentSettings: NotRequired[PaymentSettingsRequest]


class UpdatePaymentSessionRequest(TypedDict):
    amount: NotRequired[int]
    customerEmail: NotRequired[str]
    platformFee: NotRequired[int]
    splits: NotRequired[SplitPaymentsRequest]
    metadata: NotRequired[dict[str, str]]
    captureFlow: NotRequired[str]
    shippingDetails: NotRequired[ShippingDetails]
    orderDetails: NotRequired[OrderDetailsRequest]
    paymentSettings: NotRequired[PaymentSettingsRequest]


class CardDetailsRequest(TypedDict):
    number: str
    expiryMonth: str
    expiryYear: str
    cvc: NotRequired[str]
    name: NotRequired[str]


class WalletDetailsRequest(TypedDict):
    type: str
    googlePayToken: NotRequired[str]
    applePayToken: NotRequired[str]


class PaymentMethodOptionsRequest(TypedDict):
    store: bool


class BrowserDetailsRequest(TypedDict):
    acceptHeader: str
    colorDepth: int
    javaEnabled: bool
    language: str
    screenHeight: int
    screenWidth: int
    timeZoneOffset: int
    userAgent: str


class ThreeDsRequestDetails(TypedDict):
    deviceChannel: str
    browserDetails: NotRequired[BrowserDetailsRequest]


class AttemptPaymentSessionRequest(TypedDict):
    clientSecret: str
    paymentMethodType: NotRequired[str]
    cardDetails: NotRequired[CardDetailsRequest]
    walletDetails: NotRequired[WalletDetailsRequest]
    paymentMethod: NotRequired[PaymentMethodRequest]
    paymentMethodOptions: NotRequired[PaymentMethodOptionsRequest]
    billingAddress: NotRequired[Address]
    customerDetails: NotRequired[CustomerDetailsRequest]
    threeDsRequestDetails: NotRequired[ThreeDsRequestDetails]


class ThreeDsRequest(TypedDict):
    fingerprint: NotRequired[str]
    challengeResult: NotRequired[str]


class ContinuePaymentSessionRequest(TypedDict):
    clientSecret: str
    threeDs: ThreeDsRequest



from typing_extensions import List, NotRequired, TypedDict


class Customer(TypedDict):
    id: str
    email: str
    firstName: NotRequired[str]
    lastName: NotRequired[str]
    homePhoneNumber: NotRequired[str]
    mobilePhoneNumber: NotRequired[str]
    defaultPaymentMethod: NotRequired[str]
    metadata: NotRequired[dict[str, str]]
    createdTimestamp: int


class Customers(TypedDict):
    items: List[Customer]
    paginationToken: NotRequired[str]



import mimetypes
import os
import requests
from typing import Any, Optional

from ryft_sdk.models.errors import RyftError
from ryft_sdk.utils import determine_base_url

from ryft_sdk._version import __version__

default_headers = {
    "Content-Type": "application/json",
    "Accept": "application/json",
    "User-Agent": f"ryft-python-sdk/{__version__}",
    "ryft-sdk-name": "ryft-python-sdk",
    "ryft-sdk-version": __version__,
}


class RyftClient:
    def __init__(self, secret_key: Optional[str]):
        self.secret_key = secret_key or os.environ.get("RYFT_SECRET_KEY")
        if not self.secret_key:
            raise ValueError(
                "Secret API key is required. Please provide it as an argument or set the RYFT_SECRET_KEY environment variable."
            )

        self.base_url = determine_base_url(self.secret_key)
        self.session = requests.Session()

    def get(
        self,
        endpoint: str,
        params: Optional[dict[str, Any]] = None,
        account: Optional[str] = None,
    ) -> dict[str, Any]:
        return self._do("GET", endpoint, params, account=account)

    def post(
        self, endpoint: str, data: dict[str, Any], account: Optional[str] = None
    ) -> dict[str, Any]:
        return self._do("POST", endpoint, data=data, account=account)

    def patch(
        self, endpoint: str, data: dict[str, Any], account: Optional[str] = None
    ) -> dict[str, Any]:
        return self._do("PATCH", endpoint, data=data, account=account)

    def delete(
        self,
        endpoint: str,
        data: Optional[dict[str, Any]] = None,
        account: Optional[str] = None,
    ) -> dict[str, Any]:
        return self._do("DELETE", endpoint, data=data, account=account)

    def _do(
        self,
        method: str,
        endpoint: str,
        params: Optional[dict[str, Any]] = None,
        data: Optional[dict[str, Any]] = None,
        account: Optional[str] = None,
    ) -> dict[str, Any]:
        headers = default_headers
        headers["Authorization"] = self.secret_key or ""
        if account:
            headers["Account"] = account

        self.session.headers.update(headers)

        response = {}

        if method == "GET":
            response = self.session.get(f"{self.base_url}/{endpoint}", params=params)
        elif method == "POST":
            response = self.session.post(f"{self.base_url}/{endpoint}", json=data)
        elif method == "PATCH":
            response = self.session.patch(f"{self.base_url}/{endpoint}", json=data)
        elif method == "DELETE":
            response = self.session.delete(f"{self.base_url}/{endpoint}", json=data)
        else:
            raise ValueError(f"Unsupported HTTP method: {method}")

        if response.status_code < 200 or response.status_code > 299:
            raise RyftError(
                status=response.status_code,
                response=response.json(),
            )

        return response.json()

    def upload_file(
        self,
        endpoint: str,
        file_path: str,
        category: str,
        account: Optional[str] = None,
    ) -> dict[str, Any]:
        headers = default_headers
        headers["Authorization"] = self.secret_key or ""
        headers.pop("Content-Type", None)
        if account:
            headers["Account"] = account

        self.session.headers.update(headers)

        mimetypes.init()
        mimetype, _ = mimetypes.guess_type(file_path)
        if mimetype is None:
            mimetype = "application/octet-stream"

        filename = os.path.basename(file_path)

        payload = {"category": category}

        with open(file_path, "rb") as file_handle:
            files = [("file", (filename, file_handle, mimetype))]

            response = requests.request(
                "POST",
                f"{self.base_url}/{endpoint}",
                headers=headers,
                data=payload,
                files=files,
            )

            if response.status_code == 415:
                raise RyftError(
                    status=response.status_code,
                    response={
                        "message": f"Unsupported file type: {mimetype}",
                        "errors": [
                            {
                                "message": f"Unsupported file type: {mimetype}",
                                "code": "UNSUPPORTED_FILE_TYPE",
                            }
                        ],
                    },
                )

            if response.status_code < 200 or response.status_code > 299:
                raise RyftError(
                    status=response.status_code,
                    response=response.json(),
                )

            return response.json()



from typing_extensions import NotRequired, TypedDict

from ryft_sdk.models.payment_sessions.req.payment_sessions_req import (
    SplitPaymentsRequest,
)


class CapturePaymentSessionRequest(TypedDict):
    amount: NotRequired[int]
    captureType: NotRequired[str]
    platformFee: NotRequired[int]
    splits: NotRequired[SplitPaymentsRequest]


class CaptureTransactionRequest(TypedDict):
    id: str


class RefundPaymentSessionRequest(TypedDict):
    amount: NotRequired[int]
    reason: NotRequired[str]
    refundPlatformFee: NotRequired[bool]
    splits: NotRequired[SplitPaymentsRequest]
    captureTransaction: NotRequired[CaptureTransactionRequest]



from typing_extensions import List, NotRequired, TypedDict
from ryft_sdk.models.address import Address


class BankAccount(TypedDict):
    bankIdType: str
    bankId: NotRequired[str]
    accountNumberType: str
    accountNumber: str
    address: NotRequired[Address]


class PayoutMethod(TypedDict):
    id: str
    type: str
    displayName: NotRequired[str]
    currency: str
    countryCode: str
    bankAccount: BankAccount
    createdTimestamp: int
    lastUpdatedTimestamp: int


class PayoutMethods(TypedDict):
    items: List[PayoutMethod]
    paginationToken: NotRequired[str]



from typing_extensions import NotRequired, TypedDict
from ryft_sdk.models.address import Address


class BankAccountRequest(TypedDict):
    bankIdType: NotRequired[str]
    bankId: NotRequired[str]
    accountNumberType: str
    accountNumber: str
    address: NotRequired[Address]


class CreatePayoutMethodRequest(TypedDict):
    type: str
    displayName: NotRequired[str]
    currency: str
    country: str
    bankAccount: BankAccountRequest


class UpdatePayoutMethodRequest(TypedDict):
    displayName: NotRequired[str]
    bankAccount: NotRequired[BankAccountRequest]



def determine_base_url(secret_key: str) -> str:
    """
    Determine the base URL based on the secret key pattern.

    Args:
        secret_key: The Ryft API secret key

    Returns:
        The appropriate base URL for the API
    """
    if secret_key.startswith("sk_live_"):
        return "https://api.ryftpay.com/v1"
    else:
        return "https://sandbox-api.ryftpay.com/v1"



from typing_extensions import List, NotRequired, TypedDict


class TransferDestination(TypedDict):
    accountId: str


class TransferError(TypedDict):
    code: str
    description: str


class Transfer(TypedDict):
    id: str
    status: str
    amount: int
    currency: str
    reason: NotRequired[str]
    source: NotRequired[TransferDestination]
    destination: NotRequired[TransferDestination]
    errors: NotRequired[List[TransferError]]
    metadata: NotRequired[dict[str, str]]
    createdTimestamp: int
    lastUpdatedTimestamp: int


class Transfers(TypedDict):
    items: List[Transfer]
    paginationToken: NotRequired[str]



from typing import cast
from ryft_sdk.client import RyftClient
from ryft_sdk.models.account_links.dto.account_links import TemporaryAccountLink
from ryft_sdk.models.account_links.req.account_links_req import CreateTmpLinkReq


class AccountLinksClient:
    def __init__(self, client: RyftClient):
        self.client = client
        self.path = "account-links"

    async def create_tmp_link(self, req: CreateTmpLinkReq) -> TemporaryAccountLink:
        return cast(TemporaryAccountLink, self.client.post(self.path, cast(dict, req)))



from typing_extensions import NotRequired, TypedDict


class CreateCustomerRequest(TypedDict):
    email: str
    firstName: NotRequired[str]
    lastName: NotRequired[str]
    homePhoneNumber: NotRequired[str]
    mobilePhoneNumber: NotRequired[str]
    metadata: NotRequired[dict[str, str]]


class UpdateCustomerRequest(TypedDict):
    firstName: NotRequired[str]
    lastName: NotRequired[str]
    homePhoneNumber: NotRequired[str]
    mobilePhoneNumber: NotRequired[str]
    metadata: NotRequired[dict[str, str]]
    defaultPaymentMethod: NotRequired[str]



from typing_extensions import List, NotRequired, TypedDict
from ryft_sdk.models.address import AccountAddress
from ryft_sdk.models.persons.dto.persons_verification import PersonVerification


class Document(TypedDict):
    type: str
    front: str
    back: NotRequired[str]
    country: NotRequired[str]


class Person(TypedDict):
    id: str
    firstName: str
    lastName: str
    email: NotRequired[str]
    dateOfBirth: NotRequired[str]
    countryOfBirth: NotRequired[str]
    gender: str
    nationalities: List[str]
    address: AccountAddress
    phoneNumber: str
    businessRoles: List[str]
    verification: PersonVerification
    documents: List[Document]
    metadata: NotRequired[dict[str, str]]
    createdTimestamp: int
    lastUpdatedTimestamp: int


class Persons(TypedDict):
    items: List[Person]
    paginationToken: NotRequired[str]



from typing import cast, Optional
from ryft_sdk.client import RyftClient
from ryft_sdk.models.disputes.dto.disputes import Dispute, Disputes
from ryft_sdk.models.disputes.req.disputes_req import (
    AddDisputeEvidenceRequest,
    DeleteDisputeEvidenceRequest,
)


class DisputesClient:
    def __init__(self, client: RyftClient):
        self.client = client
        self.path = "disputes"

    async def list(
        self,
        startTimestamp: Optional[int] = None,
        endTimestamp: Optional[int] = None,
        ascending: Optional[bool] = None,
        limit: Optional[int] = None,
        startsAfter: Optional[str] = None,
    ) -> Disputes:
        return cast(
            Disputes,
            self.client.get(
                self.path,
                {
                    "startTimestamp": startTimestamp,
                    "endTimestamp": endTimestamp,
                    "ascending": ascending,
                    "limit": limit,
                    "startsAfter": startsAfter,
                },
            ),
        )

    async def get(self, id: str) -> Dispute:
        return cast(Dispute, self.client.get(f"{self.path}/{id}"))

    async def accept(self, id: str) -> Dispute:
        return cast(Dispute, self.client.post(f"{self.path}/{id}/accept", {}))

    async def challenge(self, id: str) -> Dispute:
        return cast(Dispute, self.client.post(f"{self.path}/{id}/challenge", {}))

    async def add_evidence(self, id: str, req: AddDisputeEvidenceRequest) -> Dispute:
        return cast(
            Dispute, self.client.patch(f"{self.path}/{id}/evidence", cast(dict, req))
        )

    async def delete_evidence(
        self, id: str, req: DeleteDisputeEvidenceRequest
    ) -> Dispute:
        return cast(
            Dispute, self.client.delete(f"{self.path}/{id}/evidence", cast(dict, req))
        )



from typing_extensions import List, Literal, NotRequired, TypedDict, Union
from ryft_sdk.models.address import AccountAddress


class AccountDocumentReq(TypedDict):
    type: str
    front: str
    back: NotRequired[str]
    country: NotRequired[str]


class AccountBusinessReq(TypedDict):
    name: str
    type: str
    registrationNumber: str
    registrationDate: NotRequired[str]
    registeredAddress: AccountAddress
    contactEmail: str
    phoneNumber: NotRequired[str]
    tradingName: NotRequired[str]
    tradingAddress: NotRequired[AccountAddress]
    tradingCountries: NotRequired[List[str]]
    websiteUrl: NotRequired[str]
    documents: NotRequired[List[AccountDocumentReq]]


class AccountIndividualReq(TypedDict):
    firstName: str
    middleNames: NotRequired[str]
    lastName: str
    email: str
    dateOfBirth: str
    countryOfBirth: NotRequired[str]
    gender: Union[Literal["Male"], Literal["Female"]]
    nationalities: List[str]
    address: AccountAddress
    phoneNumber: NotRequired[str]
    documents: NotRequired[List[AccountDocumentReq]]


class AccountPayoutScheduleReq(TypedDict):
    type: str


class AccountPayoutReq(TypedDict):
    schedule: AccountPayoutScheduleReq


class AccountSettingsReq(TypedDict):
    payouts: AccountPayoutReq


class AccountAcceptanceDetailsReq(TypedDict):
    ipAddress: str
    userAgent: NotRequired[str]
    when: NotRequired[int]


class AccountTermsOfServiceReq(TypedDict):
    acceptance: AccountAcceptanceDetailsReq


class CreateSubAccountRequest(TypedDict):
    onboardingFlow: NotRequired[str]
    email: NotRequired[str]
    entityType: NotRequired[str]
    business: NotRequired[AccountBusinessReq]
    individual: NotRequired[AccountIndividualReq]
    metadata: NotRequired[dict[str, str]]
    settings: NotRequired[AccountSettingsReq]
    termsOfService: NotRequired[AccountTermsOfServiceReq]


class UpdateBusinessReq(TypedDict):
    name: NotRequired[str]
    type: str
    registrationNumber: NotRequired[str]
    registrationDate: NotRequired[str]
    registeredAddress: NotRequired[AccountAddress]
    contactEmail: NotRequired[str]
    phoneNumber: NotRequired[str]
    tradingName: NotRequired[str]
    tradingAddress: NotRequired[AccountAddress]
    tradingCountries: NotRequired[List[str]]
    websiteUrl: NotRequired[str]
    documents: NotRequired[List[AccountDocumentReq]]


class UpdateSubAccountRequest(TypedDict):
    entityType: NotRequired[str]
    business: NotRequired[UpdateBusinessReq]
    individual: NotRequired[AccountIndividualReq]
    metadata: NotRequired[dict[str, str]]
    settings: NotRequired[AccountSettingsReq]
    termsOfService: NotRequired[AccountTermsOfServiceReq]



from typing_extensions import List, NotRequired, TypedDict
from ryft_sdk.models.payment_sessions.dto.payment_sessions import RequiredAction
from ryft_sdk.models.shipping_details import ShippingDetails


class SubscriptionCustomer(TypedDict):
    id: str


class SubscriptionPaymentSession(TypedDict):
    id: str
    clientSecret: NotRequired[str]
    requiredActions: NotRequired[RequiredAction]


class SubscriptionPaymentSessions(TypedDict):
    initial: SubscriptionPaymentSession
    latest: SubscriptionPaymentSession


class SubscriptionInterval(TypedDict):
    unit: str
    times: NotRequired[int]


class RecurringPrice(TypedDict):
    amount: int
    currency: str
    interval: SubscriptionInterval


class SubscriptionBalance(TypedDict):
    amount: int


class SubscriptionPausePaymentDetail(TypedDict):
    reason: NotRequired[str]
    resumeAtTimestamp: NotRequired[int]
    pausedAtTimestamp: int


class SubscriptionCancelDetail(TypedDict):
    reason: NotRequired[str]
    cancelledAtTimestamp: int


class SubscriptionFailureDetail(TypedDict):
    paymentAttempts: int
    lastPaymentError: str


class SubscriptionBillingDetail(TypedDict):
    totalCycles: int
    currentCycle: int
    currentCycleStartTimestamp: int
    currentCycleEndTimestamp: int
    billingCycleTimestamp: int
    nextBillingTimestamp: NotRequired[int]
    failureDetail: NotRequired[SubscriptionFailureDetail]


class StatementDescriptor(TypedDict):
    descriptor: str
    city: str


class SubscriptionPaymentSettings(TypedDict):
    statementDescriptor: NotRequired[StatementDescriptor]


class Subscription(TypedDict):
    id: str
    status: str
    description: NotRequired[str]
    customer: SubscriptionCustomer
    paymentMethod: SubscriptionCustomer
    paymentSessions: SubscriptionPaymentSessions
    price: RecurringPrice
    balance: SubscriptionBalance
    pausePaymentDetail: NotRequired[SubscriptionPausePaymentDetail]
    cancelDetail: NotRequired[SubscriptionCancelDetail]
    billingDetail: SubscriptionBillingDetail
    shippingDetails: NotRequired[ShippingDetails]
    metadata: NotRequired[dict[str, str]]
    paymentSettings: SubscriptionPaymentSettings
    createdTimestamp: int


class Subscriptions(TypedDict):
    items: List[Subscription]
    paginationToken: str



from typing import cast, Optional
from ryft_sdk.client import RyftClient
from ryft_sdk.models.deleted_resource_resp import DeletedResourceResp
from ryft_sdk.models.persons.dto.persons import Person, Persons
from ryft_sdk.models.persons.req.persons_req import (
    CreatePersonRequest,
    UpdatePersonRequest,
)


class PersonsClient:
    def __init__(self, client: RyftClient):
        self.client = client

    async def create(self, id: str, req: CreatePersonRequest) -> Person:
        return cast(Person, self.client.post(f"accounts/{id}/persons", cast(dict, req)))

    async def list(
        self,
        id: str,
        ascending: Optional[bool] = None,
        limit: Optional[int] = None,
        startsAfter: Optional[str] = None,
    ) -> Persons:
        return cast(
            Persons,
            self.client.get(
                f"accounts/{id}/persons",
                {
                    "ascending": ascending,
                    "limit": limit,
                    "startsAfter": startsAfter,
                },
            ),
        )

    async def get(self, id: str, person_id: str) -> Person:
        return cast(Person, self.client.get(f"accounts/{id}/persons/{person_id}"))

    async def update(self, id: str, person_id: str, req: UpdatePersonRequest) -> Person:
        return cast(
            Person,
            self.client.patch(f"accounts/{id}/persons/{person_id}", cast(dict, req)),
        )

    async def delete(self, id: str, person_id: str) -> DeletedResourceResp:
        return cast(
            DeletedResourceResp,
            self.client.delete(f"accounts/{id}/persons/{person_id}", {}),
        )



import hashlib
import hmac


class WebhookSignatureVerifier:
    def is_valid(self, secret: str, signature: str, payload: str) -> bool:
        return self._hmac_sha256(secret, payload) == signature

    def _hmac_sha256(self, secret: str, payload: str) -> str:
        secret_bytes = secret.encode("ascii")
        payload_bytes = payload.encode("utf-8")
        hmac_hash = hmac.new(secret_bytes, payload_bytes, hashlib.sha256)
        return hmac_hash.hexdigest()



import unittest
from ryft_sdk.webhook_signature_verifier import WebhookSignatureVerifier


class TestWebhookSignatureVerifier(unittest.TestCase):
    """Unit tests for WebhookSignatureVerifier."""

    def setUp(self):
        self.verifier = WebhookSignatureVerifier()

    def test_is_valid_should_return_false_when_signature_is_invalid(self):
        payload = '{"amount": 500, "currency": "GBP"}'
        secret_key = "abcdef4455"
        signature = "12443c521a6900579d09b1b29cf17b679f7745eb32a8018e46f44bb27103f864"

        result = self.verifier.is_valid(secret_key, signature, payload)

        self.assertFalse(result)

    def test_is_valid_should_return_true_when_signature_is_valid(self):
        payload = '{"amount": 500, "currency": "GBP"}'
        secret_key = "abcdef4455"
        secret_key = "abcdef4455"
        signature = "12443c521a6900579d09b1b29cf17b679f7745eb32a8018e46f44bb27103f865"

        result = self.verifier.is_valid(secret_key, signature, payload)

        self.assertTrue(result)



class RyftError(Exception):
    def __init__(self, status, response):
        self.requestId = response.get("requestId")
        self.code = response.get("code")
        self.status = status
        self.message = "Unknown error"
        self.errors = response.get("errors", [])

        if self.errors:
            self.message = self.errors[0].get("message", "Unknown error")

        super().__init__(self.message)
        self.__class__.__name__ = "RyftError"



from typing import cast, Optional
from ryft_sdk.client import RyftClient
from ryft_sdk.models.customers.dto.customers import Customer, Customers
from ryft_sdk.models.customers.req.customers_req import (
    CreateCustomerRequest,
    UpdateCustomerRequest,
)
from ryft_sdk.models.deleted_resource_resp import DeletedResourceResp
from ryft_sdk.models.payment_methods.dto.payment_methods_req import PaymentMethods


class CustomersClient:
    def __init__(self, client: RyftClient):
        self.client = client
        self.path = "customers"

    async def create(self, req: CreateCustomerRequest) -> Customer:
        return cast(Customer, self.client.post(self.path, cast(dict, req)))

    async def list(
        self,
        email: Optional[str] = None,
        startTimestamp: Optional[int] = None,
        endTimestamp: Optional[int] = None,
        ascending: Optional[bool] = None,
        limit: Optional[int] = None,
        startsAfter: Optional[str] = None,
    ) -> Customers:
        return cast(
            Customers,
            self.client.get(
                self.path,
                {
                    "email": email,
                    "startTimestamp": startTimestamp,
                    "endTimestamp": endTimestamp,
                    "ascending": ascending,
                    "limit": limit,
                    "startsAfter": startsAfter,
                },
            ),
        )

    async def get(self, id: str) -> Customer:
        return cast(Customer, self.client.get(f"{self.path}/{id}"))

    async def update(self, id: str, req: UpdateCustomerRequest) -> Customer:
        return cast(Customer, self.client.patch(f"{self.path}/{id}", cast(dict, req)))

    async def delete(self, id: str) -> DeletedResourceResp:
        return cast(DeletedResourceResp, self.client.delete(f"{self.path}/{id}", {}))

    async def get_payment_methods(self, id: str) -> PaymentMethods:
        return cast(
            PaymentMethods, self.client.get(f"{self.path}/{id}/payment-methods", {})
        )



from typing_extensions import Any, List, NotRequired, TypedDict


class EventCustomer(TypedDict):
    id: NotRequired[str]


class PaymentMethodTokenizedDetails(TypedDict):
    id: str
    stored: bool


class PaymentMethod(TypedDict):
    tokenizedDetails: PaymentMethodTokenizedDetails


class EventEndpoint(TypedDict):
    webhookId: str
    acknowledged: bool
    attempts: int


class Event(TypedDict):
    id: str
    eventType: str
    data: dict[str, Any]
    endpoints: List[EventEndpoint]
    accountId: NotRequired[str]
    createdTimestamp: int


class Events(TypedDict):
    items: List[Event]


class PausePaymentDetail(TypedDict):
    reason: NotRequired[str]
    resumeAtTimestamp: NotRequired[int]
    pausedAtTimestamp: int



from typing import cast, Optional
from ryft_sdk.client import RyftClient
from ryft_sdk.models.deleted_resource_resp import DeletedResourceResp
from ryft_sdk.models.subscriptions.dto.payment_session import PaymentSessions
from ryft_sdk.models.subscriptions.dto.subscriptions import Subscription
from ryft_sdk.models.subscriptions.req.subscriptions_req import (
    CreateSubscriptionRequest,
)
from ryft_sdk.models.subscriptions.req.subscriptions_req import (
    PauseSubscriptionRequest,
    UpdateSubscriptionRequest,
)


class SubscriptionsClient:
    def __init__(self, client: RyftClient):
        self.client = client
        self.path = "subscriptions"

    async def create(self, req: CreateSubscriptionRequest) -> Subscription:
        return cast(Subscription, self.client.post(self.path, cast(dict, req)))

    async def list(
        self,
        startTimestamp: Optional[int] = None,
        endTimestamp: Optional[int] = None,
        ascending: Optional[bool] = None,
        limit: Optional[int] = None,
        startsAfter: Optional[str] = None,
    ) -> Subscription:
        return cast(
            Subscription,
            self.client.get(
                self.path,
                {
                    "startTimestamp": startTimestamp,
                    "endTimestamp": endTimestamp,
                    "ascending": ascending,
                    "limit": limit,
                    "startsAfter": startsAfter,
                },
            ),
        )

    async def get(self, id: str) -> Subscription:
        return cast(Subscription, self.client.get(f"{self.path}/{id}"))

    async def update(self, id: str, req: UpdateSubscriptionRequest) -> Subscription:
        return cast(
            Subscription, self.client.patch(f"{self.path}/{id}", cast(dict, req))
        )

    async def pause(
        self, id: str, req: Optional[PauseSubscriptionRequest] = None
    ) -> Subscription:
        return cast(
            Subscription, self.client.patch(f"{self.path}/{id}/pause", cast(dict, req))
        )

    async def resume(self, id: str) -> Subscription:
        return cast(Subscription, self.client.patch(f"{self.path}/{id}/resume", {}))

    async def delete(self, id: str) -> DeletedResourceResp:
        return cast(DeletedResourceResp, self.client.delete(f"{self.path}/{id}", {}))

    async def get_payment_sessions(
        self,
        id: str,
        startTimestamp: Optional[int] = None,
        endTimestamp: Optional[int] = None,
        ascending: Optional[bool] = None,
        limit: Optional[int] = None,
        startsAfter: Optional[str] = None,
    ) -> PaymentSessions:
        return cast(
            PaymentSessions,
            self.client.get(
                f"{self.path}/{id}/payment-sessions",
                {
                    "startTimestamp": startTimestamp,
                    "endTimestamp": endTimestamp,
                    "ascending": ascending,
                    "limit": limit,
                    "startsAfter": startsAfter,
                },
            ),
        )



from typing_extensions import List, NotRequired, TypedDict

from ryft_sdk.models.address import Address
from ryft_sdk.models.shipping_details import ShippingDetails


class CustomerDetails(TypedDict):
    id: NotRequired[str]
    firstName: NotRequired[str]
    lastName: NotRequired[str]
    homePhoneNumber: NotRequired[str]
    mobilePhoneNumber: NotRequired[str]
    metadata: NotRequired[dict[str, str]]


class PaymentSessionCredentialOnFileUsage(TypedDict):
    initiator: str
    sequence: str


class PaymentSessionPreviousPayment(TypedDict):
    id: str


class RebillingDetail(TypedDict):
    amountVariance: str
    numberOfDaysBetweenPayments: int
    totalNumberOfPayments: NotRequired[int]
    currentPaymentNumber: NotRequired[int]
    expiry: NotRequired[int]


class PaymentSessionTokenizedDetails(TypedDict):
    id: str
    stored: bool


class PaymentSessionCard(TypedDict):
    scheme: str
    last4: str


class PaymentSessionWallet(TypedDict):
    type: str


class PaymentSessionChecks(TypedDict):
    avsResponseCode: NotRequired[str]
    cvvResponseCode: NotRequired[str]


class PaymentSessionPaymentMethod(TypedDict):
    type: str
    tokenizedDetails: NotRequired[PaymentSessionTokenizedDetails]
    card: NotRequired[PaymentSessionCard]
    wallet: PaymentSessionWallet
    billingAddress: NotRequired[Address]
    checks: NotRequired[PaymentSessionChecks]


class PaymentSessionFee(TypedDict):
    amount: int


class SplitPaymentDetail(TypedDict):
    id: str
    accountId: str
    amount: int
    fee: NotRequired[PaymentSessionFee]
    description: str
    metadata: NotRequired[dict[str, str]]


class SplitPaymentDetails(TypedDict):
    items: List[SplitPaymentDetail]


class StatementDescriptor(TypedDict):
    descriptor: str
    city: str


class Identify(TypedDict):
    uniqueId: NotRequired[str]
    threeDsMethodUrl: str
    threeDsMethodSignature: str
    sessionId: str
    sessionSecret: str
    threeDsMethodData: str
    scheme: str
    paymentMethodId: str


class RequiredAction(TypedDict):
    type: str
    url: NotRequired[str]
    identify: Identify


class OrderDetail(TypedDict):
    reference: str
    name: str
    quantity: int
    unitPrice: int
    taxAmount: int
    totalAmount: int
    discountAmount: int
    productUrl: NotRequired[str]
    imageUrl: NotRequired[str]


class OrderDetails(TypedDict):
    items: List[OrderDetail]


class PaymentMethodOptions(TypedDict):
    disabled: List[str]


class PaymentSettings(TypedDict):
    paymentMethodOptions: PaymentMethodOptions


class PaymentSession(TypedDict):
    id: str
    amount: int
    currency: str
    paymentType: NotRequired[str]
    entryMode: NotRequired[str]
    customerEmail: NotRequired[str]
    customerDetails: NotRequired[CustomerDetails]
    credentialOnFileUsage: NotRequired[PaymentSessionCredentialOnFileUsage]
    previousPayment: NotRequired[PaymentSessionPreviousPayment]
    rebillingDetail: NotRequired[RebillingDetail]
    enabledPaymentMethods: List[str]
    paymentMethod: NotRequired[PaymentSessionPaymentMethod]
    platformFee: NotRequired[int]
    splitPaymentDetail: SplitPaymentDetails
    status: str
    metadata: NotRequired[dict[str, str]]
    clientSecret: str
    lastError: NotRequired[str]
    refundedAmount: int
    statementDescriptor: StatementDescriptor
    requiredAction: NotRequired[RequiredAction]
    returnUrl: str
    authorizationType: NotRequired[str]
    captureFlow: NotRequired[str]
    verifyAccount: NotRequired[bool]
    shippingDetails: NotRequired[ShippingDetails]
    orderDetails: NotRequired[OrderDetails]
    paymentSettings: NotRequired[PaymentSettings]
    createdTimestamp: int
    lastUpdatedTimestamp: int


class PaymentSessions(TypedDict):
    items: List[PaymentSession]
    paginationToken: NotRequired[str]



from typing_extensions import List, NotRequired, TypedDict


class DisputeReason(TypedDict):
    code: str
    description: str


class DisputeCard(TypedDict):
    scheme: str
    last4: str


class DisputePaymentMethod(TypedDict):
    card: DisputeCard


class DisputePaymentSession(TypedDict):
    id: str
    paymentType: str
    paymentMethod: DisputePaymentMethod


class DisputeEvidenceTextEntries(TypedDict):
    billingAddress: str
    shippingAddress: str
    duplicateTransaction: str
    uncategorised: str


class DisputeFile(TypedDict):
    id: str


class DisputeEvidenceFile(TypedDict):
    proofOfDelivery: NotRequired[DisputeFile]
    customerSignature: NotRequired[DisputeFile]
    receipt: NotRequired[DisputeFile]
    shippingConfirmation: NotRequired[DisputeFile]
    customerCommunication: NotRequired[DisputeFile]
    refundPolicy: NotRequired[DisputeFile]
    recurringPaymentAgreement: NotRequired[DisputeFile]
    uncategorised: NotRequired[DisputeFile]


class DisputeEvidence(TypedDict):
    text: NotRequired[DisputeEvidenceTextEntries]
    files: NotRequired[DisputeEvidenceFile]


class DisputeCustomer(TypedDict):
    email: NotRequired[str]
    id: NotRequired[str]
    createdTimestamp: NotRequired[int]


class Dispute(TypedDict):
    id: str
    amount: int
    currency: str
    status: str
    category: str
    reason: DisputeReason
    respondBy: int
    recommendedEvidence: List[str]
    paymentSession: DisputePaymentSession
    evidence: NotRequired[DisputeEvidence]
    customer: NotRequired[DisputeCustomer]
    subAccount: NotRequired[DisputeFile]
    createdTimestamp: int
    lastUpdatedTimestamp: int


class Disputes(TypedDict):
    items: List[Dispute]
    paginationToken: NotRequired[str]



from typing import cast
from ryft_sdk.client import RyftClient
from ryft_sdk.models.deleted_resource_resp import DeletedResourceResp
from ryft_sdk.models.webhooks.dto.webhooks import CreatedWebhook, Webhook, Webhooks
from ryft_sdk.models.webhooks.req.webhooks_req import (
    CreateWebhookRequest,
    UpdateWebhookRequest,
)


class WebhooksClient:
    def __init__(self, client: RyftClient):
        self.client = client
        self.path = "webhooks"

    async def create(self, req: CreateWebhookRequest) -> CreatedWebhook:
        return cast(CreatedWebhook, self.client.post(self.path, cast(dict, req)))

    async def list(self) -> Webhooks:
        return cast(Webhooks, self.client.get(self.path))

    async def get(self, id: str) -> Webhook:
        return cast(Webhook, self.client.get(f"{self.path}/{id}"))

    async def update(self, id: str, req: UpdateWebhookRequest) -> Webhook:
        return cast(Webhook, self.client.patch(f"{self.path}/{id}", cast(dict, req)))

    async def delete(self, id: str) -> DeletedResourceResp:
        return cast(DeletedResourceResp, self.client.delete(f"{self.path}/{id}", {}))
